"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildMock = void 0;
const functions_1 = require("./utils/functions");
const fs = __importStar(require("fs"));
const path_1 = __importDefault(require("path"));
const get_installed_path_1 = require("get-installed-path");
const templateParser_1 = require("./parsers/templateParser");
const parseRouteResponse = (response, definitions) => {
    let routeResponse = '';
    if (response.constraint) {
        routeResponse = `if (${response.constraint}) {\n`;
    }
    // body can be in def(body) or file(body) -> this one is for the future if the JSON grows bigger
    if (response.body.startsWith('proxy(')) {
        routeResponse = `${routeResponse} return ${response.body} \n`;
    }
    else {
        routeResponse = `${routeResponse} return res.status(${response.statusCode}).json(${response.body}) \n`;
    }
    if (response.constraint) {
        routeResponse = `${routeResponse}}\n`;
    }
    return routeResponse;
};
const parseRouteResponses = (responses, definitions) => responses.map((response) => parseRouteResponse(response, definitions));
const parseRoute = (route, definitions) => {
    const routeContents = parseRouteResponses(route.responses, definitions).join('\n\n');
    return `router.${route.method.toLowerCase()}('${route.path}', jsonParser,  (req, res) => {${routeContents}});`;
};
const parseRoutes = (routes, definitions) => routes.map((route) => parseRoute(route, definitions));
const buildPackage = (requests, configFolder, outputFolder) => {
    const contextPath = requests.contextPath;
    const routesString = parseRoutes(requests.routes, requests.definitions).join('\n\n');
    // Create a new directory and copy template files
    fs.existsSync(outputFolder)
        ? fs.rmSync(outputFolder, { recursive: true })
        : console.log('out folder not found, copying files.');
    // Copy templates
    let templatesFolder = (0, get_installed_path_1.getInstalledPathSync)('mockatron');
    if (!templatesFolder || templatesFolder === '') {
        // In case of running mockatron from source locally
        templatesFolder = '.';
    }
    (0, functions_1.copyDir)(path_1.default.join(templatesFolder, 'templates'), outputFolder);
    (0, functions_1.copyDir)(configFolder, path_1.default.join(outputFolder, '.mockatron'));
    // Replace routes file
    let routesFileContent = fs.readFileSync(path_1.default.join(outputFolder, 'router.js'), 'utf-8');
    routesFileContent = routesFileContent.replace('// Auto generated Code', routesString);
    fs.writeFileSync(path_1.default.join(outputFolder, 'router.js'), routesFileContent);
    // Replace main file
    let indexFileContent = fs.readFileSync(path_1.default.join(outputFolder, 'index.js'), 'utf-8');
    indexFileContent = indexFileContent.replace('CONTEXT_PATH', contextPath);
    if (requests.proxy) {
        const proxyString = `app.use(proxy('${requests.proxy}'))`;
        indexFileContent = indexFileContent.replace('// PROXYING_SUPPORT', proxyString);
    }
    fs.writeFileSync(path_1.default.join(outputFolder, 'index.js'), indexFileContent);
};
const buildMock = (configFolder, outputFolder) => {
    const parsedMockatronTemplate = (0, templateParser_1.TemplateParser)(fs.readFileSync(path_1.default.join(configFolder, 'main.json'), 'utf-8'), null);
    console.log(parsedMockatronTemplate);
    buildPackage(JSON.parse(parsedMockatronTemplate), configFolder, outputFolder);
    console.log('Done.');
};
exports.buildMock = buildMock;
//# sourceMappingURL=app.js.map